local mod, mod_name, oi = Mods.new_mod("Language")

mod.dir = {
	config = "./mod_loader/config/",
	language = "./mod_loader/config/language/",
}

mod.database = {}
mod.languages = {}

-- ####################################################################################################################
-- ##### Function #####################################################################################################
-- ####################################################################################################################
mod.load = function()
	-- Load Languages
	local file = io.open(mod.dir.config .. "Language.json", "r")
	if file then
		safe_pcall(function()
			mod.languages = json.decode(file:read("*all"))
		end)
	else
		EchoConsole("[Error] No languages found.")
	end
	
	-- Load Language transelations
	for file_name in lfs.dir(mod.dir.language) do
		if string.find(file_name, "%.json$") then
			local lang_id = string.sub(file_name, 1, -6)
			local file_path = mod.dir.language .. file_name
			
			local _file = io.open(file_path, "r")
			if _file then
				mod:add_file(_file, lang_id)
			end
		end
	end
	
	-- Update widget text
	mod.widget_settings.ACTIVE.text = Localize("vmf_text_language_active_text")
end

--[[
	Generate language options based whats available in the database
]]--
mod.generate_options = function(self)
	for lang_id, _ in pairs(self.database) do
		if mod.languages[lang_id] and lang_id ~= "EN" then
			local item = {
				text = mod.languages[lang_id],
				value = lang_id
			}
			
			table.insert(self.widget_settings.ACTIVE.options, item)
		end
	end
end

--[[
	Feed the database with a translation file
]]--
mod.add_file = function(self, file, lang_id)
	local items = {}
	local db = self.database
	
	-- Check if language exist in the database
	db[lang_id] = db[lang_id]  or {}
	
	-- Load items
	safe_pcall(function()
		items = json.decode(file:read("*all"))
	end)
	
	--merge the items in the database
	for key, value in pairs(items) do
		db[lang_id][tostring(key)] = value
	end
end

--[[
	Transelation data can have a array and parameters.
]]--
mod.to_str = function(data, ...)
	local str = data
	local args = table.pack(...)
	
	-- Add enters on each string part
	if type(data) == "table" then
		str = table.concat(data, "\n")
	end
	
	-- Add arguments in the string with pcall
	if #args > 0 then
		safe_pcall(function()	
			str = string.format(str, unpack(args))
		end)
	end
	
	return str
end