local mod, mod_name, oi = Mods.new_mod("network")

mod._calls = {}

-- ####################################################################################################################
-- ##### Function #####################################################################################################
-- ####################################################################################################################
--[[
	Register rpc calls
]]--
mod.register = function(name, callback)
	if type(name) ~= "string" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "register", "#1", "string"))
		return
	end
	
	if type(callback) ~= "function" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "register", "#2", "function"))
		return
	end
	
	mod._calls[name] = callback
end

--[[
	Get the rpc function
]]--
mod.get = function(name)
	return mod._calls[name]
end

--[[
	Transelated all known transmission calls and proxy them through the chat channel 2
]]--
mod.send_rpc = function (rpc_name, peer_id, ...)
	local args = table.pack(...)
	
	Managers.state.network.network_transmit:send_rpc(
		"rpc_chat_message", peer_id, 2, Network.peer_id(), rpc_name, table.serialization(args), false, true, false
	)
end

mod.send_rpc_server = function(rpc_name, ...)
	local args = table.pack(...)
	
	Managers.state.network.network_transmit:send_rpc_server(
		"rpc_chat_message", 2, Network.peer_id(), rpc_name, table.serialization(args), false, true, false
	)
end

mod.send_rpc_clients = function (rpc_name, ...)
	local args = table.pack(...)
	
	Managers.state.network.network_transmit:send_rpc_clients(
		"rpc_chat_message", 2, Network.peer_id(), rpc_name, table.serialization(args), false, true, false
	)
end

mod.send_rpc_clients_except = function (rpc_name, except, ...)
	local args = table.pack(...)
	
	Managers.state.network.network_transmit:send_rpc_clients_except(
		"rpc_chat_message", except, 2, Network.peer_id(), rpc_name, table.serialization(args), false, true, false
	)
end

mod.send_rpc_all = function(rpc_name, ...)
	local args = table.pack(...)
	
	Managers.state.network.network_transmit:send_rpc_all(
		"rpc_chat_message", 2, Network.peer_id(), rpc_name, table.serialization(args), false, true, false
	)
end

mod.send_rpc_all_except = function(rpc_name, except, ...)
	local args = table.pack(...)
	
	Managers.state.network.network_transmit:send_rpc_all_except(
		"rpc_chat_message", except, 2, Network.peer_id(), rpc_name, table.serialization(args), false, true, false
	)
end
