local mod, mod_name, oi = Mods.new_mod("option_menu")
--[[
	Option Menu:
		The OptionInjector must be draw at a fixed order to show
		the option menu at the right way. Because the mods are loaded
		in a none fixed order this must be controlled by a option_menu.
		
		option_menu allows you to register groups and items to manage the order. 
]]--

mod._groups = {}
mod._conf = {}

local template_group_widget = {
	["save"] = "vmf_unknown",
	["widget_type"] = "checkbox",
	["text"] = "<Unknown>",
	["default"] = false,
	["hide_options"] = {
		{
			true,
			mode = "show",
			options = {},
		},
		{
			false,
			mode = "hide",
			options = {},
		},
	},
}
	
--[[
	Create Group
	
	This allows you to create group and item options.
	The group option is always a check box that show/hide all items.
]]--
mod.add_group = function(self, id, text)
	-- Check id
	if type(id) ~= "string" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "add_group", "#1", "string"))
		return
	end
	
	-- Check text
	if type(text) ~= "string" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "add_group", "#2", "string"))
		return
	end

	if self:_get_group(id) == nil then
		table.insert(self._groups, {id = id, text = text, hide_options = {}})
	end
end

--[[
	Get the group with selected id
]]--
mod._get_group = function(self, id)
	if self._groups then
		for _, group in ipairs(self._groups) do
			if group.id == id then
				return group
			end
		end
	end
end

--[[
	Add a item to a group with widget setting
]]--
mod.add_item = function(self, group_id, setting, hide_option)
	-- Check group_id
	if type(group_id) ~= "string" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "add_group", "#1", "string"))
		return
	end
	
	-- Check setting
	if type(setting) ~= "table" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "add_group", "#2", "table"))
		return
	end
	
	-- Check hide_option
	if hide_option and type(hide_option) ~= "boolean" then
		Mods.show_error(Localize("vmf_text_core_error_var_type", "add_group", "#3", "boolean"))
		return
	end

	-- Check settings exist
	if not self._conf[group_id] then
		self._conf[group_id] = {}
	end
	
	-- Insert new settings in group
	table.insert(self._conf[group_id], setting)
	
	-- Show and hide influence
	if hide_option then
		local group = self:_get_group(group_id)
		if not group then
			self:add_group(group_id, group_id)
			group = self:_get_group(group_id)
		end
		table.insert(group.hide_options, setting.save)
	end
end

--[[
	Draw the option menu
]]--
mod.draw = function(self)
	for _, group in ipairs(self._groups) do
		group.setting = table.clone(template_group_widget)
		group.setting.save = "vmf_" .. group.id
		group.setting.text = group.text
		
		if self._conf[group.id] then
			-- Create widget
			oi.CreateWidget(group.setting)
			for _, setting in ipairs(self._conf[group.id]) do
				if setting.save ~= group.setting.save then
					-- Create widget
					local widget = oi.CreateWidget(setting)
					
					if setting.cursor_offset then
						widget.style.tooltip_text.cursor_offset = table.clone(setting.cursor_offset)
					end
				else
					Mods.show_error(Localize("vmf_text_option_menu_same_name"))
					EchoConsole(Localize("vmf_text_option_menu_same_name_advice", setting.save))
				end
			end
			-- Draw Seperator
			Mods.exec("action", "insert_option_seperator")
		end
		
		-- set show/hide settings
		group.setting.hide_options[1].options = group.hide_options
		group.setting.hide_options[2].options = group.hide_options
	end
end