Mods = {
	_before_reload = {},
	settings = {},
	saved_reqs = {}, 

	init = function()
		-- Load Plugins
		Mods.add_plugins()

		-- Load Settings
		Mods.load_settings()
		
		-- Load Functions
		Mods.exec("function", "Global")
		Mods.exec("function", "Table")
		Mods.exec("function", "OptionsInjector")
		Mods.exec("function", "Chat")
		Mods.exec("function", "Hook")
		Mods.exec("function", "Debug")
		Mods.exec("function", "Gui")
		Mods.exec("function", "Ui")
		Mods.exec("function", "Network")
		Mods.exec("function", "Keyboard")
		Mods.exec("function", "FFI")
		Mods.exec("function", "Manager")
		Mods.exec("function", "OptionMenu")
		Mods.exec("function", "Mutators")
		Mods.exec("function", "Language")
		
		--Option menu groups
		Mods.option_menu:add_group("mod_settings", "Mod Settings")
		Mods.option_menu:add_group("system", "System")
		Mods.option_menu:add_group("system_chat", "System Chat")
		Mods.option_menu:add_group("mutators", "Mutators")
		Mods.option_menu:add_group("cheats", "Cheats")
		Mods.option_menu:add_group("items", "Items")
		Mods.option_menu:add_group("spawning", "Spawning")
		Mods.option_menu:add_group("movement", "Movement")
		Mods.option_menu:add_group("hud", "HUD")
		
		-- Run core scripts and hooks
		Mods.exec("patch/core", "Language")
		Mods.exec("patch", "ModSettings")
		Mods.exec("patch/core", "SystemChat")
		Mods.exec("patch/core", "Manager")
		Mods.exec("patch/core", "UI")
		Mods.exec("patch/core", "Gui")
		Mods.exec("patch/core", "Chat")
		Mods.exec("patch/core", "OptionsInjector")
		Mods.exec("patch/core", "Keyboard")
		Mods.exec("patch/core", "Network")
		Mods.exec("patch/core", "Mutators")
		Mods.exec("patch/core", "OptionMenu")
		
		-- Fixes
		Mods.exec("patch/fix", "Backend")
		Mods.exec("patch/fix", "Fire Grenade")
		Mods.exec("patch/fix", "Fireflame")
		Mods.exec("patch/fix", "Breed")
		Mods.exec("patch/fix", "Books")
		Mods.exec("patch/fix", "Animation")
		Mods.exec("patch/fix", "Dialog")
		
		-- Execute mods
		Mods.manager:run()
		
		-- Future mods that needs to be migrated
		Mods.exec("patch", "Fatshark")
		--Mods.exec("patch/unstable", "SwitchCharacter")
		--Mods.exec("patch/unstable", "Menu")
		
		EchoConsole(Localize("vmf_text_core_start", Mods.settings.version))
	end,
	
	load_settings = function()
		local settings_file = io.open("mod_loader/config/Settings.json", "r")
		if settings_file then
			pcall(function()
				local settings = json.decode(settings_file:read("*all"))
				
				Mods.settings = settings
			end)
			settings_file:close()
		end
	end,
	
	before_reload = function(self, func)
		if type(func) == "function" then
			-- Add Function
			table.insert(self._before_reload, func)
		end
	end,
	
	reload = function(self)
		-- Execute all functions before reloading
		if self._before_reload then
			for _, func in ipairs(self._before_reload) do
				func()
			end
		end
		
		Mods.exec("", "mod_loader")
	end,

	--[[
		Assignes a namespace for a new mod and returns it alongside mod name and OptionsInjector
	]]--
	new_mod = function(mod_name)
		-- Check mod_name
		if type(mod_name) ~= "string" then
			Mods.show_error(Localize("vmf_text_core_error_var_type", "new_mod", "#1", "string"))
			return
		end
	
		-- Check mod already exist
		if Mods[mod_name] then
			Mods.show_error(Localize("vmf_text_core_error_mod_exist", mod_name))
			return
		end
		
		-- Create mod space
		Mods[mod_name] = {
			get = Application.user_setting,
			set = Application.set_user_setting,
			save = Application.save_user_settings
		}
		
		-- Return results
		return Mods[mod_name], mod_name, OptionsInjector
	end,

	--[[
		Returns mod's table alongside mod name and OptionsInjector
	]]--
	get_mod = function(mod_name)
		-- Return mod information
		return Mods[mod_name], mod_name, OptionsInjector
	end,
	
	--[[
		Execute a lua file/mod file
	]]--
	exec = function(group, file_name, args)
		-- file path
		local file_path = "mod_loader/" .. file_name .. ".lua"
		if group and group ~= "" then
			file_path = "mod_loader/" .. group .. "/" .. file_name .. ".lua"
		end
		
		-- Open file
		local file = io.open(file_path, "r")

		-- if file path not exist, check if mod file exist
		if not file and Mods.manager then
			local config = Mods.manager:get(group)
			if config then
				file_path = "mods/" .. config.mod_file .. ".mod"

				-- Open mod file
				local zfile, zfile_err = zip.open(file_path)
				if zfile then
					file_path = file_name .. ".lua"
					file = zfile:open(file_path)
				end
			end
		end

		-- If file exist then execute file
		if file then
			local status, value = Mods.exec_file(file, args)
			if not status then
				EchoConsole("Error on file '" .. file_path .. "'")
				EchoConsole(value)
			end
			
			-- Close file
			file:close()
			
			return status, value
		end

		EchoConsole("File not found '" .. file_path .. "'")
		
		return false
	end,

	--[[
		Loads lua file and caches return value to avoid loading the same thing multiple times
	]]--
	require = function(group, file_name)
		if Mods.saved_reqs[group] and Mods.saved_reqs[group][file_name] then
			return Mods.saved_reqs[group][file_name]
		end
		if not Mods.saved_reqs[group] then
			Mods.saved_reqs[group] = {}
		end
		local status, value = Mods.exec(group, file_name)
		Mods.saved_reqs[group][file_name] = status and value
		return status and value
	end,
	
	--[[
		Execute a lua file
	]]--
	exec_file = function(file, args)
		return pcall(function()
			-- Read lua code
			local data = file:read("*all")
			
			-- Convert string to a function
			local func = loadstring(data)
			
			-- Execute function
			local value = func(unpack(args or {}))
			
			return value
		end)
	end,
	
	--[[
		Add Mods/plugins directory to package
	]]--
	add_plugins = function()
		-- ZIP
		local f = assert(package.loadlib([[.\mod_loader\plugins\zip.dll]], "luaopen_zip"))
		f()  -- actually open the library
		require("zip")
		
		-- LFS
		local f = assert(package.loadlib([[.\mod_loader\plugins\lfs.dll]], "luaopen_lfs"))
		f()  -- actually open the library
		require("lfs")
		
		-- JSON
		Mods.exec("plugins", "json")
		require("json")
	end,
	
	show_error = function(text)
		local error_base_text = "[" .. Localize("vmf_text_core_error"):upper() .. "]"
		
		EchoConsole(error_base_text .. " " .. text)  
	end,
}

-- Initialize Mods
Mods.init()