local mod, mod_name, oi = Mods.get_mod("Language")

mod.widget_settings = {
	ACTIVE = {
		["save"] = "vmf_language_country",
		["widget_type"] = "dropdown",
		["text"] = "Language",
		["tooltip"] =  "",
		["value_type"] = "string",
		["options"] = {
			{text = "English", value = "EN"},
		},
		["default"] = 1, -- Default first option is enabled. In this case Off
	},
}

mod.on_enter_language = nil

-- ####################################################################################################################
-- ##### Options ######################################################################################################
-- ####################################################################################################################
mod.create_options = function()
	Mods.option_menu:add_item("mod_settings", mod.widget_settings.ACTIVE, true)
end

-- ####################################################################################################################
-- ##### Hook #########################################################################################################
-- ####################################################################################################################
--[[
	Transelate mod text_id to text
]]--
Mods.hook.set(mod_name, "Localize", function(func, text_id, ...)
	local text = func(text_id)
	
	-- If fatshark Localize couldnt resolve the text_id
	if text == "<" .. text_id .. ">" then
		local db = mod.database
		
		if db then
			local lang_id = mod.get(mod.widget_settings.ACTIVE.save)
			
			-- Check setting language
			if db[lang_id] and db[lang_id][text_id] then
				return mod.to_str(db[lang_id][text_id], ...)
			end
			
			-- Check english because this must be always exist
			if db["EN"] and db["EN"][text_id] then
				return mod.to_str(db["EN"][text_id], ...)
			end
		end
	end
	
	return text
end)

Mods.hook.set(mod_name, "OptionsView.on_enter", function(func, ...)
	mod.on_enter_language = mod.get(mod.widget_settings.ACTIVE.save)
	
	func(...)
end)

Mods.hook.set(mod_name, "OptionsView.on_exit", function(func, ...)
	func(...)
	
	if mod.on_enter_language ~= mod.get(mod.widget_settings.ACTIVE.save) then
		Mods.exec("action", "reload")
	end
end)

-- ####################################################################################################################
-- ##### Start ########################################################################################################
-- ####################################################################################################################
mod.create_options()
mod.load()

--[[
	Generate the language options when all mods are loaded
]]--
Mods.manager:after_run(function()
	mod:generate_options()
end)